/* Popup Menu with Submenus Example */

#include <Xm/Xm.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeBG.h>
#include <Xm/RowColumn.h>


/*********** Callback for the Pushbuttons ***********************/

void ButtonCB (w, client_data, call_data) 
Widget	w;		/*  widget id		*/
caddr_t	client_data;	/*  data from application   */
caddr_t	call_data;	/*  data from widget class  */
{
   /*  print message and terminate program  */
   printf ("Button %s selected.\n", client_data);
	
}

/************** Event Handler for Popup Menu ************************/

PostIt (w, popup, event)
Widget w;
Widget popup;
XButtonEvent * event;
{
   if (event->button != Button3)
	return;
   XmMenuPosition(popup, event);
   XtManageChild(popup);
}


/***********************Main Logic for Program ********************/


void main (argc, argv)
int argc;
char **argv;
{
   Widget toplevel, popup, rc;
   Widget submenu1, submenu2, submenu3, buttons[2];
   Widget popupBtn[2], sub1Btn[3], sub2Btn[2], sub3Btn[2];
   Arg args[5];
   XmString string;


/* Initialize toolkit */
   toplevel = XtInitialize (argv[0], "PopupMenu", NULL, 0, &argc, argv);

/* Create RowColumn in toplevel with two pushbuttons */

   XtSetArg(args[0], XmNwidth, 400);
   XtSetArg(args[1], XmNheight, 125);
   XtSetArg(args[2], XmNresizeWidth, False);
   XtSetArg(args[3], XmNresizeHeight, False);
   XtSetArg(args[4], XmNadjustLast, False);
   rc = XmCreateRowColumn(toplevel, "rc", args, 5);  
   XtManageChild(rc);

   buttons[0] = XmCreatePushButtonGadget(rc, "button1", NULL, 0);
   XtAddCallback(buttons[0], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"1");

   buttons[1] = XmCreatePushButtonGadget(rc, "button2", NULL, 0);
   XtAddCallback(buttons[1], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"2");
   XtManageChildren(buttons, 2);

/* Create popup menu */

   popup = XmCreatePopupMenu(rc, "popup", NULL, 0);
   XtAddEventHandler(rc, ButtonPressMask, False, (XtEventHandler)PostIt, (XtPointer)popup);

/* Create two submenus and CascadeButtons in the popup menu */

   submenu1 = (Widget)XmCreatePulldownMenu(popup, "submenu1", NULL, 0);

   string = XmStringCreate("First Submenu", XmSTRING_DEFAULT_CHARSET); 
   XtSetArg(args[0], XmNlabelString, string);
   XtSetArg(args[1], XmNsubMenuId, submenu1);
   popupBtn[0] = XmCreateCascadeButtonGadget(popup, "cbutton1", args, 2);
   XmStringFree(string);   


   submenu2 = (Widget)XmCreatePulldownMenu(popup, "submenu2", NULL, 0);

   string = XmStringCreate("Second Submenu", XmSTRING_DEFAULT_CHARSET); 
   XtSetArg(args[0], XmNlabelString, string);
   XtSetArg(args[1], XmNsubMenuId, submenu2);
   popupBtn[1] = XmCreateCascadeButtonGadget(popup, "cbutton2", args, 2);
   XmStringFree(string);   
   XtManageChildren(popupBtn, 2);

/* Create pushbuttons in submenu1 and submenu2. */

   sub1Btn[0] = XmCreatePushButtonGadget(submenu1, "button1a", NULL, 0);
   XtAddCallback(sub1Btn[0], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"1a");

   sub1Btn[1] = XmCreatePushButtonGadget(submenu1, "button1b", NULL, 0);
   XtAddCallback(sub1Btn[1], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"1b");

   sub2Btn[0] = XmCreatePushButtonGadget(submenu2, "button2a", NULL, 0);
   XtAddCallback(sub2Btn[0], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"2a");

   sub2Btn[1] = XmCreatePushButtonGadget(submenu2, "button2b", NULL, 0);
   XtAddCallback(sub2Btn[1], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"2b");
   XtManageChildren(sub2Btn, 2);

/* Create a submenu of submenu 1 */

   submenu3 = (Widget)XmCreatePulldownMenu(submenu1, "submenu3", NULL, 0);

   string = XmStringCreate("To Their Submenu", XmSTRING_DEFAULT_CHARSET); 
   XtSetArg(args[0], XmNlabelString, string);
   XtSetArg(args[1], XmNsubMenuId, submenu3);
   sub1Btn[2] = XmCreateCascadeButtonGadget(submenu1, "cbutton3", args, 2);
   XmStringFree(string);   
   XtManageChildren(sub1Btn, 3);

/* Create pushbuttons in submenu 3 */

   sub3Btn[0] = XmCreatePushButtonGadget(submenu3, "button3a", NULL, 0);
   XtAddCallback(sub3Btn[0], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"3a");

   sub3Btn[1] = XmCreatePushButtonGadget(submenu3, "button3b", NULL, 0);
   XtAddCallback(sub3Btn[1], XmNactivateCallback, (XtCallbackProc)ButtonCB, (XtPointer)"3b");
   XtManageChildren(sub3Btn, 2);

/* Get and dispatch events */

   XtRealizeWidget(toplevel);

   XtMainLoop();
}


